import os, sys
import pygame
import random
import math
import ConfigParser
from pygame.locals import *

if not pygame.font: print 'Warning, fonts disabled'
if not pygame.mixer: print 'Warning, sound disabled'

fontname = "manga_speak.ttf"
configfile = "config2.txt"
scoretable="score.txt"
height = 600
width = 800

def load_image(filename): 
    fullname = os.path.join('images', filename)
    try:
        image = pygame.image.load(fullname)
    except pygame.error, message:
        print 'Cannot load image:', filename
        raise SystemExit, message
    image = image.convert_alpha()
    return image

def switch_screen(next_screen, *args):
    global current
    current = next_screen(*args)

def add_points(points):
    current.levelpoints += points
    current.totalpoints += points

class Configs:
    def __init__(self, filename):
        self.config = ConfigParser.ConfigParser()
        self.config.readfp(open(filename))
    
    def has_section(self, section):
        return self.config.has_section(section)
    
    def get_dict(self, section):
        if (self.config.has_section(section)):
            idict = dict(self.config.items(section))
            for key in idict:
                idict[key] = eval(idict[key])
            return idict
        return False

class Screen:
    def update(self):
        pygame.display.update()

    def tick(self, time):
        pass

    def handleevents(self, event):
        if event.type == QUIT:
            quit()
        elif event.type == KEYDOWN and event.key == K_ESCAPE:
            quit()
        elif event.type == KEYDOWN and event.key == K_TAB:
            switch_screen(BossScreen)

class BossScreen(Screen):
    def __init__(self):
        self.background = load_image("spreadsheet3.gif")
        self.stuff = 1
    def update(self):
        global screen
        if self.stuff == 1:
            screen = pygame.display.set_mode((929, 891), pygame.NOFRAME)
            screen.blit(self.background, (0,0))
            pygame.display.update() 
            self.stuff = 0
    def handleevents(self, event):
        if event.type == KEYDOWN and event.key == K_TAB:
            screen = pygame.display.set_mode((800, 600))
            switch_screen(MainMenuScreen)
            
class LevelScreen(Screen):
    def __init__(self):
        self.levelpoints = 0
        self.totalpoints = 0
        self.n = 0
        self.levels = 1
        self.crosshair = Crosshair()
        self.sprites = pygame.sprite.RenderPlain(self.crosshair)
        self.otters = pygame.sprite.RenderPlain()
        self.effects = pygame.sprite.RenderPlain()
        self.topbar = load_image("bar.png")
        
        conf = Configs(configfile)
        self.level = [Level(ottergroup = self.otters, **conf.get_dict("level1"))]
        while conf.has_section("level"+str(self.levels+1)):
            self.levels += 1
            self.level.append(Level(ottergroup = self.otters, **conf.get_dict("level"+str(self.levels))))
        self.elapsed = 0
        self.currentlevel = self.level[self.n]
        pygame.mouse.set_visible(0)
        self.playing = 1
        
    def update(self):
        if self.playing:
            self.sprites.update()
            self.otters.update()
            self.effects.update()
            screen.blit(self.currentlevel.background, (0, 0))
            self.otters.draw(screen)
            self.sprites.draw(screen)
            self.effects.draw(screen)

            screen.blit(self.topbar, (0,0))

            font = pygame.font.Font(fontname, 36)
            text = font.render("Points: " + str(self.levelpoints) + " | Time left: " + str(self.getTimer(False)), 1, (10, 10, 10))
            textpos = text.get_rect(topright=(screen.get_width(),0))
            screen.blit(text, textpos)
        elif self.elapsed >= 2000:
            self.playing = 1
            self.elapsed = 0            
        pygame.display.update()

    def tick(self, time):
        self.elapsed += time
        self.currentlevel.checkOtter(time)

        if self.elapsed/1000 >= self.currentlevel.timelimit:
            self.levelup()

    def getTimer(self, elapsed=1):
        if elapsed:
            return self.elapsed/1000
        else:
            return self.currentlevel.timelimit - self.elapsed/1000

    def handleevents(self,event):
        Screen.handleevents(self,event)
        if event.type == MOUSEBUTTONDOWN:
            self.crosshair.shoot(self.otters, self.effects)

    def levelup(self):
       if self.levelpoints >= self.currentlevel.pointreq and self.n+1 < self.levels:
            """ a level up screen """
            font = pygame.font.Font(fontname, 36)
            background = load_image("main.jpg")
            title = load_image("levelup.png")
            titlepos = title.get_rect(midtop=(screen.get_width()/2,150))
            text1 = font.render("Level " + str(self.n+2),1,(0,0,0))
            textpos1 = text1.get_rect(midtop=(screen.get_width()/2,250))
            text2 = font.render("Points:",1,(0,0,0))
            textpos2 = text2.get_rect(topright=(350,350))
            text3 = font.render("Level " + str(self.n+1) + ":",1,(0,0,0))
            textpos3 = text3.get_rect(topright=(400,390))
            text4 = font.render("Total: ", 1, (0,0,0))
            textpos4 = text4.get_rect(topright=(400,430))
            text5 = font.render(str(self.levelpoints), 1, (0,0,0))
            textpos5 = text5.get_rect(topright=(500,390))
            text6 = font.render(str(self.totalpoints), 1, (0,0,0))
            textpos6 = text6.get_rect(topright=(500,430))
            screen.blit(background, (0,0))
            screen.blit(title, titlepos)
            screen.blit(text1, textpos1)
            screen.blit(text2, textpos2)
            screen.blit(text3, textpos3)
            screen.blit(text4, textpos4)
            screen.blit(text5, textpos5)
            screen.blit(text6, textpos6)
            pygame.display.update()

            self.currentlevel = self.level[self.n + 1]
            self.elapsed = 0
            self.playing = 0
            self.levelpoints = 0
            self.n += 1
            self.otters.empty()
            self.effects.empty()

       else:
           self.fp = open(scoretable)
           newlist = []
           for line in self.fp:
               oldscore = line.strip().split(',')
               newlist.append((oldscore[0],int(oldscore[1])))
           if self.totalpoints > newlist[2][1]:
               switch_screen(ScoreInputScreen, self.totalpoints)
               print "'%s' was entered" % current.ask("Name")
           else:
                switch_screen(GameOverScreen,self.totalpoints)


class Button(pygame.sprite.Sprite):
    def __init__ (self, pos, im, function=None, *args):
        pygame.sprite.Sprite.__init__(self)
        self.image = load_image(im)
        self.rect = self.image.get_rect()
        self.rect.center = pos
        self.function = function
        self.args = args 
        self.pos = pos #pos is a tuple

    def draw(self):
        screen.blit(self.image , self.rect)

    def buttonpress(self):
        if event.type == MOUSEBUTTONDOWN:
            self.function(*self.args)

class GameOverScreen(Screen):
   def __init__(self,points):
        self.points = points
        self.background = load_image("main.jpg")
        self.over = load_image("gameover.png")
        font = pygame.font.Font(fontname, 72)
        self.text1 = font.render("Total score: " + str(self.points),2, (0,0,0))
        self.overpos = self.over.get_rect(midtop=(screen.get_width()/2,150))
        self.textpos1 = self.text1.get_rect(midtop=(screen.get_width()/2,250))
        self.button1 = Button((250,400) , "menu.jpg", switch_screen, MainMenuScreen)
        self.button2 = Button((550,400) , "quit.jpg", quit)
        self.buttons = pygame.sprite.Group(self.button1, self.button2)
        pygame.mouse.set_visible(1)

    
   def handleevents(self,event):
        Screen.handleevents(self,event)
        if event.type == MOUSEBUTTONDOWN and self.button1.rect.collidepoint(pygame.mouse.get_pos()):
            self.button1.buttonpress()
        elif event.type == MOUSEBUTTONDOWN and self.button2.rect.collidepoint(pygame.mouse.get_pos()):
            self.button2.buttonpress()

   def update(self):
        screen.blit(self.background, (0,0))
        screen.blit(self.text1, self.textpos1)
        screen.blit(self.over, self.overpos)
        self.buttons.update()
        self.buttons.draw(screen)
        pygame.display.update()

class InstructionScreen(Screen):
    def __init__(self):
        font = pygame.font.Font(fontname, 26)
        text = """The otters have taken over the world! \n\
        Well, not quite, but it is up to you to keep their \n\
        population under control. Click on the moving otters to \n\
        shoot them and gain points before the time is up. You \n\
        need a certain number of points to go to the next level. \n\
        The boss otter (with the tophat) gives you extra points \n\
        but needs to be shot twice. You will lose points if you \n\
        .    shoot a fake otter (with the smiley face)."""
        text = text.split('\n')
        self.text = []
        for line in text:
            self.text.append(font.render(line.lstrip(), 1, (0,0,0)))
        
        self.ins = load_image("instructions.png")
        self.inspos = self.ins.get_rect(midtop=(screen.get_width()/2, 50))
        self.button1 = Button((400,500) , 'menu.jpg', switch_screen, MainMenuScreen)
        self.background = load_image("main.jpg")
        self.buttons = pygame.sprite.Group(self.button1)
        pygame.mouse.set_visible(1)
    
    def handleevents(self,event):
        Screen.handleevents(self,event)
        if event.type == MOUSEBUTTONDOWN and  self.button1.rect.collidepoint(pygame.mouse.get_pos()):
            self.button1.buttonpress()            
        
    def update(self):
        self.buttons.update()
        screen.blit(self.background, (0,0))
        screen.blit(self.ins, self.inspos)
        for i in range(len(self.text)):
            screen.blit(self.text[i], (60, 150+36*i))
        self.buttons.draw(screen)
        pygame.display.update()
        
class MainMenuScreen(Screen):
    def __init__(self):
        font = pygame.font.Font(fontname, 16)
        self.logo = load_image("logo.png")
        self.button1 = Button((400,400) , "start.jpg", switch_screen, LevelScreen)
        self.button2 = Button((400,500) , "instructions.jpg", switch_screen, InstructionScreen)
        self.background = load_image("main.jpg")
        self.logopos = self.logo.get_rect(midtop=(screen.get_width()/2,75))
        self.buttons = pygame.sprite.Group(self.button1, self.button2)
        pygame.mouse.set_visible(1)
    
        self.text1 = font.render("Created by: Elena Oleynikova, Jared Barrow, and Chujiao Ma.",2, (0,0,0))
        self.textpos1 = self.text1.get_rect(midtop=(screen.get_width()/2,550))

    def handleevents(self,event):
        Screen.handleevents(self,event)
        if event.type == MOUSEBUTTONDOWN and self.button1.rect.collidepoint(pygame.mouse.get_pos()):
            self.button1.buttonpress()
        elif event.type == MOUSEBUTTONDOWN and self.button2.rect.collidepoint(pygame.mouse.get_pos()):
            self.button2.buttonpress()

    def update(self):
        self.buttons.update()
        screen.blit(self.background, (0,0))
        screen.blit(self.logo, self.logopos)
        screen.blit(self.text1, self.textpos1)
        self.buttons.draw(screen)
        pygame.display.update()

class Otter(pygame.sprite.Sprite):
    def __init__(self, pos=(100, 100), speed=(5,0)):
        pygame.sprite.Sprite.__init__(self)
        self.image = load_image("otter.png")
        self.rect = self.image.get_rect()
        self.rect.topleft = list(pos)
        self.move = list(speed)
        self.area = screen.get_rect()
        self.pointvalue = abs(10*speed[0])
        onscreen = self.on_screen()
        if onscreen != (0,0):
            self.rect.topleft = (self.rect.top - onscreen[0]*self.rect.height, self.rect.left - onscreen[1]*self.rect.width)

    def on_screen(self):
        ret = []
        if self.rect.left < self.area.left:
            ret.append(-1)

        elif self.rect.right > self.area.right:
            ret.append(1)
        else:
            ret.append(0)
        if self.rect.top < self.area.top:
            ret.append(-1)
        elif self.rect.bottom > self.area.bottom:
            ret.append(1)
        else:
            ret.append(0)
        return tuple(ret)

    def update(self):
        self.shuffle()

    def shuffle(self):
        newpos = self.rect.move(self.move)
        self.rect = newpos
        if self.on_screen() != (0,0):
            self.kill()

    def get_shot(self):
        add_points(self.pointvalue)
        self.kill()
        return True
        
class Boss(Otter):
    def __init__(self, pos=(100,100), speed=(5,0)):
        Otter.__init__(self, pos, speed)
        self.image = load_image("boss_otter.png")
        self.rect = self.image.get_rect()
        self.rect.topleft = pos
        self.pointvalue = 3*abs(20*speed[0])
        self.maxhit = 2
        self.wounds = 0
        
    def get_shot(self):
        if self.wounds < self.maxhit:
            self.wounds += 1
            return False
        elif self.wounds == self.maxhit:
            add_points(self.pointvalue)
            self.kill()
            return True

class OtterSuit(Otter):
    def __init__(self, pos=(100, 100), speed=(5,0)):
        Otter.__init__(self, pos, speed)
        self.image = load_image("otter_suit.png")
        self.rect = self.image.get_rect()
        self.rect.topleft = pos
        self.pointvalue = -100
        onscreen = self.on_screen()
        if onscreen != (0,0):
            self.rect.topleft = (self.rect.top - onscreen[0]*self.rect.height, self.rect.left - onscreen[1]*self.rect.width)

    def on_screen(self):
        ret = []
        if self.rect.left < self.area.left:
            ret.append(-1)
        elif self.rect.right > self.area.right:
            ret.append(1)
        else:
            ret.append(0)
        if self.rect.top < self.area.top:
            ret.append(-1)
        elif self.rect.bottom > self.area.bottom:
            ret.append(1)
        else:
            ret.append(0)
        return tuple(ret)

    def get_shot(self):
        add_points(self.pointvalue)
        self.kill()
        return True

        
class Crosshair(pygame.sprite.Sprite):
    def __init__(self):
        pygame.sprite.Sprite.__init__(self)
        self.image = load_image("crosshair.png")
        self.rect = self.image.get_rect()

    def update(self):
        self.rect.center = pygame.mouse.get_pos()

    def shoot(self, group, effects):
        shotlist = pygame.sprite.spritecollide(self, group, False)
        if len(shotlist):
            for shot in shotlist:
                if shot.get_shot():
                    effects.add(ShowText(shot.rect.midtop, shot.pointvalue,color=(255,215,0)))
                    effects.add(Explosion(shot.rect.center))
                else:
                    effects.add(Explosion(self.rect.center, "small", 10))

class ShowText(pygame.sprite.Sprite):
    def __init__(self, pos, text, size=36, fade=10, color=(0,0,0)):
        pygame.sprite.Sprite.__init__(self)
        font = pygame.font.Font(fontname, size)
        self.image = font.render(str(text), 1, color)
        self.rect = self.image.get_rect()
        self.rect.midbottom = pos
        self.counter = fade

    def update(self):
        self.counter -= 1
        if self.counter == 0:
            self.kill()

class Explosion(pygame.sprite.Sprite):
    def __init__(self, pos, explosiontype="big", fade=20):
        pygame.sprite.Sprite.__init__(self)
        self.image = load_image(explosiontype + "explosion.png")
        self.rect = self.image.get_rect()
        self.rect.center = pos
        self.counter = fade

    def update(self):
        self.counter -= 1
        if self.counter == 0:
            self.kill()

class Level:
    def __init__(self, ottergroup, timelimit, ottertypes, spawn, xspeed, yspeed, pointreq, background):
        self.timelimit = timelimit
        if type(ottertypes) == list:
            self.ottertypes = ottertypes
        else:
            self.ottertypes = [ottertypes]
        self.spawn = list(spawn)
        self.xspeed = list(xspeed)
        self.yspeed = list(yspeed)
        self.tto = []
        for thing in self.spawn:
            self.tto.append(random.randint(1000*thing[0],1000*thing[1]))
        self.ottergroup = ottergroup
        self.pointreq = pointreq
        self.background = load_image(background)

    def checkOtter(self, time):
        for i in range(len(self.tto)):
            if self.tto[i] < 0.1:
                self.spawnOtter(i)
            self.tto[i] -= time

    def spawnOtter(self, i):
        pos = self.randomLocation()
        speed = self.randomSpeed(pos,i)
        self.ottergroup.add(self.ottertypes[i](pos, speed))
        self.tto[i] = random.randint(1000*self.spawn[i][0], 1000*self.spawn[i][1])

    def randomSpeed(self, pos, i):
        screensize = screen.get_rect()
        speed = [random.randint(self.xspeed[i][0], self.xspeed[i][1]), random.randint(self.yspeed[i][0], self.yspeed[i][1])]
        direction = [random.choice([-1,1]), random.choice([-1,1])]
        if (pos[0] < screensize.right * .25):
            direction[0] = 1
        elif (pos[0] > screensize.right * .75):
            direction[0] = -1
        if (pos[1] < screensize.bottom * .25):
            direction[1] = 1
        elif (pos[1] > screensize.bottom * .75):
            direction[1] = -1
        speed[0] *= direction[0]
        speed[1] *= direction[1]
        return speed

    def randomLocation(self):
        screensize = screen.get_rect()
        location = (random.randint(screensize.left, screensize.right), random.randint(screensize.top, screensize.bottom))
        return location

class ScoreInputScreen(Screen):
    def __init__(self,points):
        self.points = points
        self.background = load_image("main.jpg")
        self.size = 36
        self.font = pygame.font.Font(fontname, self.size)
        self.text1 = self.font.render("You qualifies for the high score table!",2, (0,0,0))
        self.textpos1 = self.text1.get_rect(midtop=(screen.get_width()/2,100))
        self.text2 = self.font.render("Enter your name then press 'enter'.",2, (0,0,0))
        self.textpos2 = self.text2.get_rect(midtop=(screen.get_width()/2,150))
        self.text3 = self.font.render("Total score: " + str(self.points),2, (0,0,0))
        self.textpos3 = self.text3.get_rect(midtop=(screen.get_width()/2,200))
        self.fgcolor = (255,255,255)
        self.bgcolor = (0,0,0)
        pygame.mouse.set_visible(1)
        screen.blit(self.background, (0,0))
        screen.blit(self.text1, self.textpos1)
        screen.blit(self.text2, self.textpos2)
        screen.blit(self.text3, self.textpos3)
        pygame.display.update()

    def display_box(self, message):
        "Print a message in a box in the middle of the screen"
        area = Rect(0, 0, 500, self.size+15)
        area.center = screen.get_rect().center
        pygame.draw.rect(screen, self.bgcolor, area, 0)
        pygame.draw.rect(screen, self.fgcolor, area, 1)
        oldclip = screen.get_clip()
        screen.set_clip(area.inflate(-4, -4))
        if len(message):
            text = self.font.render(message, 1, self.fgcolor)
            screen.blit(text, area.move(5, 4))
        screen.set_clip(oldclip)
        pygame.display.update(area)
    
    def ask(self, question):
        "ask(screen, question) -> answer"
	current_string = ''
        self.display_box(question + ":")
	self.loop = 1
        while self.loop == 1 :
            event = pygame.event.wait()
            if event.type == QUIT:
                quit()
            if event.type != KEYDOWN:
                continue
            if event.key == K_BACKSPACE:
                current_string = current_string[:-1]
            elif event.key == K_RETURN:
                switch_screen(HighScoreScreen, self.points, current_string)	
		print 'did it switch?'
                self.loop = 0
            elif event.unicode:
                current_string += event.unicode
            self.display_box(question + ": " + current_string)

class HighScoreScreen(Screen):
    def __init__(self, points, name):

        self.points = points
        self.name=name
        self.fp = open(scoretable)
        newlist = []
        add = 0
        for line in self.fp:
            oldscore = line.strip().split(',')
            newlist.append((oldscore[0],int(oldscore[1])))

        for i in range(len(newlist)):
            if self.points > newlist[i][1]:
                if add <1:
                    newlist.insert(i, (str(self.name), self.points))
                    add+=1
        newlist=newlist[:3]
        print newlist, type(newlist)
        
        rewrite = open(scoretable,'w')
        for each in newlist:
            rewrite.write(each[0] + "," + str(each[1]) + "\n")
        rewrite.close()

        self.background = load_image("main.jpg")
        self.over = load_image("highscore.jpg")
        self.overpos = self.over.get_rect(midtop=(screen.get_width()/2,100))

        font = pygame.font.Font(fontname, 37)
        self.text1 = font.render(newlist[0][0]+": "+str(newlist[0][1]),2, (0,0,0))
        self.textpos1 = self.text1.get_rect(midtop=(screen.get_width()/2,200))
        self.text2 = font.render(newlist[1][0]+": "+str(newlist[1][1]),2, (0,0,0))
        self.textpos2 = self.text2.get_rect(midtop=(screen.get_width()/2,250))
        self.text3 = font.render(newlist[2][0]+": "+str(newlist[2][1]),2, (0,0,0))
        self.textpos3 = self.text3.get_rect(midtop=(screen.get_width()/2,300))
        self.button1 = Button((250,450) , "menu.jpg", switch_screen, MainMenuScreen)
        self.button2 = Button((550,450) , "quit.jpg", quit)
        self.buttons = pygame.sprite.Group(self.button1, self.button2)
        pygame.mouse.set_visible(1)
    
    def handleevents(self,event):
        Screen.handleevents(self,event)
        if event.type == MOUSEBUTTONDOWN and self.button1.rect.collidepoint(pygame.mouse.get_pos()):
            self.button1.buttonpress()
        elif event.type == MOUSEBUTTONDOWN and self.button2.rect.collidepoint(pygame.mouse.get_pos()):
            self.button2.buttonpress()

    def update(self):
        screen.blit(self.background, (0,0))
        screen.blit(self.text1, self.textpos1)
        screen.blit(self.text2, self.textpos2)
        screen.blit(self.text3, self.textpos3)
        screen.blit(self.over, self.overpos)
        self.buttons.update()
        self.buttons.draw(screen)
        pygame.display.update()
        


pygame.init()
screen = pygame.display.set_mode((width, height))
pygame.display.set_caption('Shoot Some Otters! Version Expo')
clock = pygame.time.Clock()
current = MainMenuScreen()
pygame.display.update()


while 1:
    milliseconds = clock.tick(30)
    current.tick(milliseconds)
    for event in pygame.event.get():
        if event.type == QUIT:
            quit()
        elif event.type == KEYDOWN and event.key == K_ESCAPE:
            quit()
        else:
            current.handleevents(event)
    current.update()
